@@ -10,6 +10,7 @@ import bitmath
 import boto3
 import ec2_metadata
 import psutil as psutil
+import pylogconf.core
 from hurry.filesize import size
 from pylogconf.core import create_pylogconf_file
 from pytconf import register_endpoint, register_function_group
@@ -18,9 +19,7 @@ import pyflexebs
 import pyflexebs.version
 from pyflexebs.configs import ConfigAlgo, ConfigProxy
 
-import pypathutil.common
-
-from pyflexebs.utils import run_with_logger, get_logger, check_root
+from pyflexebs.utils import run_with_logger, get_logger, check_root, configure_proxy, check_tools, dump
 
 GROUP_NAME_DEFAULT = "default"
 GROUP_DESCRIPTION_DEFAULT = "all pyflexebs commands"
@@ -56,51 +55,54 @@ def daemon() -> None:
     """
     Run daemon and monitor disk utilization
     """
-    logger = get_logger()
-    configure_proxy()
-    check_tools()
-    check_root()
-    metadata = ec2_metadata.ec2_metadata
-    instance_id = metadata.instance_id
-    ec2_resource = boto3.resource('ec2', region_name=metadata.region)
-    instance = ec2_resource.Instance(instance_id)
-    tags = instance.tags
-    ec2_client = boto3.client('ec2', region_name=metadata.region)
+    with daemon.DaemonContext():
+        pylogconf.core.setup_systemd(name="pyflexebs")
+        logger = get_logger()
+        logger.info("starting")
+        configure_proxy()
+        check_tools()
+        check_root()
+        metadata = ec2_metadata.ec2_metadata
+        instance_id = metadata.instance_id
+        ec2_resource = boto3.resource('ec2', region_name=metadata.region)
+        instance = ec2_resource.Instance(instance_id)
+        tags = instance.tags
+        ec2_client = boto3.client('ec2', region_name=metadata.region)
 
-    volumes = instance.volumes.all()
-    device_to_volume = dict()
-    for volume in volumes:
-        for a in volume.attachments:
-            device = normalize_device(a["Device"])
-            device_to_volume[device] = volume
+        volumes = instance.volumes.all()
+        device_to_volume = dict()
+        for volume in volumes:
+            for a in volume.attachments:
+                device = normalize_device(a["Device"])
+                device_to_volume[device] = volume
 
-    while True:
-        logger.info("checking disk utilization")
-        for p in psutil.disk_partitions():
-            if p.mountpoint in ConfigAlgo.disregard:
-                continue
-            if p.fstype not in ConfigAlgo.file_systems:
-                continue
-            if TAG_DONT_RESIZE in tags:
-                continue
-            logger.info("checking {} {} {}".format(
-                p.device,
-                p.mountpoint,
-                p.fstype,
-            ))
-            if ConfigAlgo.watermark_max is not None:
-                if psutil.disk_usage(p.mountpoint).percent >= ConfigAlgo.watermark_max:
-                    logger.info("max watermark detected at disk {} mountpoint {}".format(
-                        p.device,
-                        p.mountpoint,
-                    ))
-                    logger.info("percent is {}, total is {}, used is {}".format(
-                        psutil.disk_usage(p.mountpoint).percent,
-                        psutil.disk_usage(p.mountpoint).total,
-                        psutil.disk_usage(p.mountpoint).used,
-                    ))
-                    enlarge_volume(p, device_to_volume, ec2_client)
-        time.sleep(ConfigAlgo.interval)
+        while True:
+            logger.info("checking disk utilization")
+            for p in psutil.disk_partitions():
+                if p.mountpoint in ConfigAlgo.disregard:
+                    continue
+                if p.fstype not in ConfigAlgo.file_systems:
+                    continue
+                if TAG_DONT_RESIZE in tags:
+                    continue
+                logger.info("checking {} {} {}".format(
+                    p.device,
+                    p.mountpoint,
+                    p.fstype,
+                ))
+                if ConfigAlgo.watermark_max is not None:
+                    if psutil.disk_usage(p.mountpoint).percent >= ConfigAlgo.watermark_max:
+                        logger.info("max watermark detected at disk {} mountpoint {}".format(
+                            p.device,
+                            p.mountpoint,
+                        ))
+                        logger.info("percent is {}, total is {}, used is {}".format(
+                            psutil.disk_usage(p.mountpoint).percent,
+                            psutil.disk_usage(p.mountpoint).total,
+                            psutil.disk_usage(p.mountpoint).used,
+                        ))
+                        enlarge_volume(p, device_to_volume, ec2_client)
+            time.sleep(ConfigAlgo.interval)
 
 
 def normalize_device(dev: str) -> str:
@@ -217,16 +219,6 @@ def show_volumes() -> None:
         dump(v)
 
 
-def dump(obj):
-    """
-    debugging function to dump objects
-    """
-    for attr in dir(obj):
-        if attr.startswith("__"):
-            continue
-        print("obj.%s = %r" % (attr, getattr(obj, attr)))
-
-
 @register_endpoint(
     group=GROUP_NAME_DEFAULT,
 )
@@ -269,29 +261,96 @@ def show_policies() -> None:
         logger.info("policy_arn: [{}]".format(policy_arn))
 
 
-def configure_proxy():
-    if ConfigProxy.http_proxy is not None:
-        os.environ['http_proxy'] = ConfigProxy.http_proxy
-        os.environ['HTTP_PROXY'] = ConfigProxy.http_proxy
-    if ConfigProxy.https_proxy is not None:
-        os.environ['https_proxy'] = ConfigProxy.https_proxy
-        os.environ['HTTPS_PROXY'] = ConfigProxy.https_proxy
-    if ConfigProxy.no_proxy is not None:
-        os.environ['no_proxy'] = ConfigProxy.no_proxy
-        os.environ['NO_PROXY'] = ConfigProxy.no_proxy
+SYSTEMD_FOLDER = "/lib/systemd/system"
+SERVICE_NAME = "pyflexebs.service"
+UNIT_FILE = "/lib/systemd/system/{}".format(SERVICE_NAME)
+
+CONTENT = """[Unit]
+Description=pyflexebs service
+After=multi-user.target
+
+[Service]
+Type=simple
+ExecStart={}
+
+[Install]
+WantedBy=multi-user.target
+"""
 
 
-def check_tools():
+@register_endpoint(
+    group=GROUP_NAME_DEFAULT,
+)
+def service_install() -> None:
     """
-    Check that the command line tools we need are available
+    Install the service on the current machine
     """
-    logger = get_logger()
-    problems = False
-    for app in ["xfs_growfs", "resize2fs"]:
-        if pypathutil.common.find_in_standard_path(app) is None:
-            problems = True
-            logger.error("please install executable [{}]".format(app))
-    if problems:
-        sys.exit(1)
+    check_root()
+    assert os.path.isdir(SYSTEMD_FOLDER), "systemd folder does not exist. What kind of linux is this?"
+    assert not os.path.isfile(UNIT_FILE), "you already have the service installed"
+    abs_path_to_program = sys.argv[0]
+    if not os.path.isabs(abs_path_to_program):
+        abs_path_to_program = os.path.join(os.getcwd(), abs_path_to_program)
+    with open(UNIT_FILE, "wt") as f:
+        f.write(CONTENT.format(abs_path_to_program))
+    subprocess.check_call([
+        "systemctl",
+        "daemon-reload",
+    ])
+    # subprocess.check_call([
+    #    "systemctl",
+    #    "start",
+    #    SERVICE_NAME,
+    # ])
 
 
+@register_endpoint(
+    group=GROUP_NAME_DEFAULT,
+)
+def service_uninstall() -> None:
+    """
+    Uninstall the service from the current machine
+    """
+    check_root()
+    assert os.path.isdir(SYSTEMD_FOLDER), "systemd folder does not exist. What kind of linux is this?"
+    assert os.path.isfile(UNIT_FILE), "you dont have the service installed"
+    # subprocess.check_call([
+    #    "systemctl",
+    #    "stop",
+    #    SERVICE_NAME,
+    # ])
+    os.unlink(UNIT_FILE)
+    subprocess.check_call([
+        "systemctl",
+        "daemon-reload",
+    ])
+
+
+@register_endpoint(
+    group=GROUP_NAME_DEFAULT,
+)
+def service_start() -> None:
+    """
+    Start the service
+    """
+    check_root()
+    subprocess.check_call([
+        "systemctl",
+        "start",
+        SERVICE_NAME,
+    ])
+
+
+@register_endpoint(
+    group=GROUP_NAME_DEFAULT,
+)
+def service_stop() -> None:
+    """
+    Stop the service
+    """
+    check_root()
+    subprocess.check_call([
+        "systemctl",
+        "stop",
+        SERVICE_NAME,
+    ])
